﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace XDGNoDuplicatesCase
{
    public class ViewModel
    {
        public ObservableCollection<SampleData> Data { get; set; }
        public List<string> Names { get; set; }

        string[] names = "Andrew;Jen;Eric;Alex;Pamela;Mike;Valerie;Steve".Split(';');

        public ViewModel()
        {
            Data = new ObservableCollection<SampleData>();

            Names = names.ToList();

            for(int i=0; i < names.Length - 1; i++)
            {
                Data.Add(new SampleData()
                {
                    ID = i,
                    Name = names[i]
                });
            }
        }
    }

    public class SampleData : IDataErrorInfo, INotifyPropertyChanged
    {
        private string _dataError;
        private Dictionary<string, string> _dataErrors = new Dictionary<string, string>();

        #region SampleData Properties
        private string _name;
        public string Name
        {
            get { return _name; }
            set
            {
                if(_name != value)
                {
                    _name = value;
                    RaisePropertyChanged("Name");
                }
            }
        }

        private int _id;
        public int ID
        {
            get { return _id; }
            set
            {
                if(_id != value)
                {
                    _id = value;
                    RaisePropertyChanged("ID");
                }
            }
        }
        #endregion

        public void RaiseDuplicateName(bool raiseNotification)
        {
            string error = "Duplicate Name Detected!";           
            this.SetFieldDataError("Name", error, raiseNotification);
        }

        #region IDataErrorInfo Implementation
        public string GetFieldDataError(string fieldName)
        {
            string error;
            _dataErrors.TryGetValue(fieldName, out error);

            return error;
        }

        public string GetRecordDataError()
        {
            return _dataError;
        }

        public bool HasDataError()
        {
            return !string.IsNullOrEmpty(_dataError)
                || null != _dataErrors && _dataErrors.Count > 0;
        }


        public void SetFieldDataError(string fieldName, string error, bool raiseNotification)
        {
            if (string.IsNullOrEmpty(error))
            {
                _dataErrors.Remove(fieldName);
                
                if (0 == _dataErrors.Count)
                    this.SetRecordDataError(null, true);
            }
            else
            {
                _dataErrors[fieldName] = error;

                this.SetRecordDataError("Error", true);
            }

            if (raiseNotification)
                this.RaisePropertyChanged(fieldName);
        }

        public void SetRecordDataError(string error, bool raiseNotification)
        {
            if (_dataError != error)
            {
                _dataError = error;

                if (raiseNotification)
                    this.RaisePropertyChanged("");
            }
        }

        string IDataErrorInfo.Error
        {
            get
            {
                return _dataError;
            }
        }

        string IDataErrorInfo.this[string fieldName]
        {
            get
            {
                return this.GetFieldDataError(fieldName);
            }
        }
        #endregion

        #region INotifyPropertyChanged Members
        private void RaisePropertyChanged(string propName)
        {
            if (null != _propertyChangedDelegate)
                _propertyChangedDelegate(this, new PropertyChangedEventArgs(propName));
        }

        private PropertyChangedEventHandler _propertyChangedDelegate;

        event PropertyChangedEventHandler INotifyPropertyChanged.PropertyChanged
        {
            add
            {
                _propertyChangedDelegate = (PropertyChangedEventHandler)Delegate.Combine(_propertyChangedDelegate, value);
            }
            remove
            {
                _propertyChangedDelegate = (PropertyChangedEventHandler)Delegate.Remove(_propertyChangedDelegate, value);
            }
        }
        #endregion  
    }
}
